

import stamp.core.*;
import com.micromegacorp.math.hardware.*;

class TestSHT11
{
  static Float32 m_fValue = new Float32();
  static Float32 m_fValue2 = new Float32();
  static Float32 m_fValue3 = new Float32();
  static Float32 m_fValue4 = new Float32();

  static Float32 m_fTemperature = new Float32();

  static Timer m_Timer = new Timer();
  static StringBuffer m_StringBuffer = new StringBuffer();

  static final int DATA_PIN = CPU.pin4;
  static final int CLOCK_PIN = CPU.pin5;

  static final int shtTemp = 0x0003;
  static final int shtHumi = 0x0005;
  static final int shtStatW = 0x0006;
  static final int shtStatR = 0x0007;
  static final int shtReset = 0x001E;

  static final int Ack = 0;
  static final int NoAck = 1;


  public TestSHT11()
  {


  }

  public static void main()
  {

    // reset the uM-FPU and get version
    if (!Fpu.reset()) {
      System.out.println("uM-FPU not responding.");
      return;
    }
    System.out.println(Fpu.version());


    while ( true )
    {
      shtConnectionReset();       // Reset device connection

      SleepSHT11();

      MeasureTemp();         // Measure the temperature
      MeasureHumidity();

      CPU.delay( 5000 );  // Long delay

      System.out.print('\u0010');  // Clears the terminal
    }
  }

  // This function makes the Javelin to wait ~1/4 seconds
  static void SleepSHT11()
  {
    CPU.writePin( DATA_PIN, true );
    CPU.setInput( DATA_PIN );
    m_Timer.mark();

    while (!m_Timer.timeout(250))
    {
      boolean bValue = CPU.readPin( DATA_PIN );
      if ( !bValue )
        return;
    }

  }



  static void shtConnectionReset()
  {
    CPU.writePin( CLOCK_PIN, false );
    // connection reset: 9 clock cyles with ShtData high, then start sequence
    CPU.shiftOut( DATA_PIN, CLOCK_PIN, 9, CPU.SHIFT_LSB, 0xFFF );

    // generates SHT11 "start" sequence
    shtStart();
  }

  // Function initializes the SHT11 chip
  // generates SHT11 "start" sequence
  //           _____         _____
  // ShtData        |_______|
  //              ___     ___
  // Clock    ___|   |___|   |___
  static void shtStart()
  {
    CPU.setInput( DATA_PIN );
    CPU.writePin( CLOCK_PIN, false );
    CPU.writePin( CLOCK_PIN, true );
    CPU.writePin( DATA_PIN, false );
    CPU.writePin( CLOCK_PIN, false );
    CPU.writePin( CLOCK_PIN, true );
    CPU.setInput( DATA_PIN );
    CPU.writePin( CLOCK_PIN, false );
  }
  static void MeasureHumidity()
  {
    shtStart(); // Alert device

    int nAckBit = shtWriteByte( shtHumi ); // send humidity command
    SleepSHT11(); // wait for measurement

    // Get the humidity
    int nHumidity = shtReadByte( Ack ) << 8; // Get the MSB
    nHumidity |= shtReadByte( NoAck );

    // Linearize humidity:
    // m_fValue = (soRH * 0.0405) - (soRH * 0.002 * soRH * 0.0014) - 4
    m_fValue.set( nHumidity );
    m_fValue2.set( "0.0405" );
    m_fValue.multiply( m_fValue2 );

    m_fValue2.set( nHumidity );
    m_fValue3.set( "0.002" );

    m_fValue2.multiply( m_fValue3 );
    m_fValue2.multiply( nHumidity );
    m_fValue3.set( "0.0014" );
    m_fValue2.multiply( m_fValue3 );
    m_fValue2.subtract( 4 );
    m_fValue.subtract( m_fValue2 );

    // temperature compensated humidity
    // m_fValue2 = (tc - 25) * (soRH * 0.00008 + 0.01) + m_fValue
    m_fValue2.set( m_fTemperature );
    m_fValue2.subtract( 25 );

    m_fValue3.set( nHumidity );
    m_fValue4.set("0.00008");
    m_fValue3.multiply( m_fValue4 );
    m_fValue4.set("0.01");
    m_fValue3.add( m_fValue4 );

    m_fValue2.multiply( m_fValue3 );
    m_fValue2.add( m_fValue);

    // m_fValue2 is "rhTrue"
    // m_fValue is "rhLin"

    // Make % values
    // m_fValue2.divide( 10 );
    // m_fValue.divide( 10 );

    System.out.println( m_fValue.toString() );
    System.out.println( m_fValue2.toString() );
  }

  static void MeasureTemp()
  {
    shtStart(); // Alert device
    int nAckBit = shtWriteByte( shtTemp ); // send temperature command
    SleepSHT11(); // wait for measurement

    // Get the temperature
    int nTemperature = shtReadByte( Ack ) << 8; // Get the MSB
    nTemperature |= shtReadByte( NoAck );

    // Calculate the temperature celsius
    m_fValue.set( nTemperature );
    // System.out.println( m_fValue.toString() );
    m_fValue2.set("0.01");
    m_fValue.multiply( m_fValue2 );
    // System.out.println( m_fValue.toString() );
    m_fValue.subtract( 40 );
    m_fValue.multiply( 10 );
    m_fValue2.set( m_fValue );
    int nTemp = Float32Math.round( m_fValue2 );

    m_fTemperature.set( nTemp );
    m_fTemperature.divide( 10 );
    String str = m_fTemperature.toString();
    m_StringBuffer.clear();

    int nIndex = str.indexOf(".");
    if ( nIndex > 0 ) // The "." could be found
    {
      // Copy only the string until "." + one character
      for ( int i = 0; i <= nIndex + 1; i++ )
        m_StringBuffer.append( str.charAt( i ) );
    }
    else
    {
	m_StringBuffer.append( str );
    }

    System.out.println( m_StringBuffer.toString() );
  }

  static int shtReadByte( int nAcknowledgeBit )
  {
    CPU.writePin( CLOCK_PIN, false );   // Prepare for shiftIn command
    int nData = CPU.shiftIn( DATA_PIN, CLOCK_PIN, 8, CPU.PRE_CLOCK_MSB ); // PRE_CLOCK_MSB

    CPU.writePin( CLOCK_PIN, false );   // Prepare for shiftIn command
    CPU.shiftOut( DATA_PIN, CLOCK_PIN, 8, CPU.SHIFT_LSB, nAcknowledgeBit );    // Send the acknowledge bit
    CPU.setInput( DATA_PIN ); // release data line

    return ( nData );
  }
  // Function writes 8 bits to the SHT11
  static int shtWriteByte( int data )
  {
    CPU.writePin( CLOCK_PIN, false );
    data = data << 8;
    CPU.shiftOut( DATA_PIN, CLOCK_PIN, 8, CPU.SHIFT_MSB, data );

    CPU.writePin( CLOCK_PIN, false );
    int nData = CPU.shiftIn( DATA_PIN, CLOCK_PIN, 1, CPU.PRE_CLOCK_LSB ); // PRE_CLOCK_LSB

    return ( nData );
  }

}